<TeXmacs|0.3.4.3>

<style|article>

<\body>
  <assign|code|<macro|body|<surround|<vspace*|1fn>|<vspace|1fn><format|no
  indentation after>|<with|font family|tt|language|verbatim|<arg|body>>>>><ap\
  ply|title|Towards a Mathemagix compiler><apply|author|Joris van der
  Hoeven><apply|address|Laboratoire de Mathmatiques (bt. 425)<format|next
  line>Universit Paris-Sud<format|next line>91405 Orsay Cedex<format|next
  line>France>

  <apply|maketitle>

  <\abstract>
    In this paper we describe an experimental implementation of a Mathemagix
    compiler in C++ and we specify how the generated code should look like.
  </abstract>

  <section|Survey of the compiler>

  <subsection|Overview of the main compilation steps>

  <\description>
    <expand|item*|Parsing.>The ASCII source program is transformed into a
    lisp-like expression.

    <expand|item*|Semantic analysis.>This is the key step of the compilation.
    A hashtable is created which associates to each <with|font
    shape|italic|symbol> occurring in the lisp expression a list of
    <with|font shape|italic|meanings>. These meanings contain in particular
    the types of the symbols and possible export modifiers. In the lisp
    expression itself each symbol is replaced by its meaning at that place.
    The implicit type conversion which were made are also inserted. After
    this stage, the program has been proved to be semantically correct.

    <expand|item*|Inlining.>The lisp expression is rewritten a second time
    into another lisp expression in which all inline exports have been
    expanded and in which all specializations of generic code in conditional
    exports have been made. Other abstract optimizations might also be
    performed at this stage.

    <expand|item*|Code generation.>As a result of the inlining step, we end
    up with a scheme expression which reflects exactly the structure of the
    program as we want it to be generated in memory. The code generation step
    merely has to generate the corresponding C or assembly code.
  </description>

  <subsection|Overview of the implementation>

  The mathemagix compiler depends on the <tt|gc> garbage collector and
  GNU/Bison for the generation of a parser. The <tt|src> directory contains
  the source code of the Mathemagix compiler, which is subdivided in the
  following parts:

  <\description>
    <expand|item*|<tt|types.>>This directory contains the main data types
    used during compile time. The main type is <tt|generic>; each other type
    may be converted to this type. The other types comprise strings, symbols,
    arrays and lists of generics, etc.

    <expand|item*|<tt|large.>>This directory contains all routines for file
    handling, error generation and dealing with large programs.

    <expand|item*|<tt|parser.>>Consists of the Mathemagix grammer in
    <tt|src/mm_parser.y> and all routines for dealing with lexical analysis
    and parsing.

    <expand|item*|<tt|compiler.>>This directory contains the core of the
    compiler. It consists of a certain number of general purpose routines in
    the main directory and a specific file for most of the Mathemagix
    keywords in the <tt|compiler/Keywords> subdirectory.

    <expand|item*|<tt|runtypes.>>Some additional runtime types for an
    experimental interpreter.

    <expand|item*|<tt|interpreter.>>This directory contains an experimental
    Mathemagix interpreter. More precisely, the source code is first compiled
    and the generated intermediate lisp program is interpreted.
  </description>

  When running <tt|make>, the executables <tt|mm-lex>, <tt|mm-parse>,
  <tt|mm-compile> and <tt|mm-run> are produced in order to respectively test
  the lexical analysis, parser, compiler and interpreter. Some sample input
  files for these executables can be found in <tt|samples>. The <tt|mmx>
  directory will contain standard Mathemagix programs.

  <section|The parser>

  The mathemagix parser transforms a source string into a ``lisp
  expression'', which is either a <tt|list> of other lisp expressions or a
  <tt|symbol>. Actually, a <tt|symbol> is a hashed <tt|generic>, usually a
  <tt|string>. In other words, a symbol is just a pointer to a generic, with
  the property that two symbols which point to the same generic use the same
  pointer. In particular, symbols can be compared just by comparing the
  pointers and any additional occurence of a symbol necessitates a constant
  memory storage.

  All (non <tt|nil>) <tt|generic> objects come with a <tt|begin> and <tt|end>
  location in the source code (which may be <tt|-1>, if such a location does
  not exist). The standard routines on lists and other basic types keep track
  of these source location in a ``clever'' way. The <tt|begin> and <tt|end>
  locations are actually integers, but the file manipulation routines in the
  <tt|large> directory provide support for associating precise locations in
  the individual files of the source code to these integers.

  It might be interesting to also implement an inverse of the Mathemagix
  parser, i.e.<inactive|<group|>> a program capable of computing a Mathemagix
  program from a lisp-expression. Such a program would for instance be useful
  for debugging optimization through rewriting or symbolic manipulation of
  Mathemagix expressions.

  <section|The main compilation step>

  <subsection|The idea of lazy compilation>

  The Mathemagix compiler is based on the concept of lazy compilation.
  Concretely speaking, this means that the lisp source program is first
  transformed into a lazy expression (of type <tt|lazy>). Such a lazy
  expression is an object with methods for partial compilation, error
  generation and access of the context (the <tt|context> type). Additional
  fields may contain the lazy subexpressions of the expression.

  Now let <tt|prg> be the main lazy expression corresponding to the whole
  program. In order to compile the program, we first call the partial
  compilation method of <tt|prg>. This method recursively launches the
  compilation of all subexpressions (certain parts might actually be compiled
  in parallel). Not all these recursive partial compilations can necessarily
  be achieved in one pass, due to potential mutual dependencies.
  Nevertheless, if one of the recursive partial compilations did progress,
  then the partial compilation method of <tt|prg> returns <tt|true>. If so,
  we keep on calling the partial compilation method of <tt|prg>, until
  compilation is complete.

  In presence of invalid mutual dependencies, the compilation may block
  before completing. In that case, the main compiler again calls the partial
  compilation method of <tt|prg>, but with the <tt|force> flag set to
  <tt|true>. This allows blocking subexpressions to progress, while issuing
  appropriate error messages.

  <subsection|The current environment>

  The current environment of a lazy expression contains a table with a list
  of meanings for each innermost variable symbol and a parent environment for
  the remaining variable symbols. Each meaning of a symbol gives rise to a
  unique instance of the <tt|mirror> class. The main aim of the compiler is
  to disambiguate each symbol in the source program, by associating the
  appropriate meaning (mirror) to it. For instance, in the program

  <\code>
    x:String;

    x:Integer;

    f(x:Integer):Integer == x*x;
  </code>

  three mirrors for <tt|x> will be created, which correspond to each possible
  meaning of <tt|x> in the different contexts. New meanings of symbols can be
  introduced in the following ways:

  <\enumerate>
    <item>Definition of symbols in the global environment.

    <item>Definition of exports in a class definition or implementation.

    <item>Importation or inheritance of symbols from a class.

    <item>Arguments of functions inside the function body.

    <item>Local variables inside blocks.

    <item>As a consequence of the <with|font family|tt|with <with|font
    shape|italic|variable> do> construct.
  </enumerate>

  <subsection|The different stages of compilation>

  At the first compilation stage, we create the lazy expression corresponding
  to the program, using the <tt|make_lazy> routine and it's variants. The
  lazy expressions provide the <tt|compile> method for partial compilation.
  This method recursively compiles all lazy subexpressions in the <tt|subs>
  list. When done, the abstract <tt|compile_this> method is called, which
  needs to be provided by any concrete lazy keyword class.

  Inside the <tt|compile_this> method, the results of the compilations of the
  lazy subexpressions may be examined and further processed. For instance,
  the <tt|get_value> method gets the value of the compiled expression and
  <tt|get_type> it's type. Similarly, for each possible meaning <tt|x> of
  type <tt|T> of the lazy expression being compiled, the method
  <tt|meaning(x,T)> is called. In particular, the result of a partial
  compilation may be ambiguous, the ambiguity being resolved by the parent
  expression. The method <tt|dereference> may be called in order to add the
  results of all implicit conversions of the form <tt|alias T -\<gtr\> T> to
  the list of meanings. The method <tt|filter> may be used to get the meaning
  of a particular type (and discard all others).

  <subsection|Importation>

  <subsection|Type satisfaction>

  <subsection|Implicit type conversion>

  <section|Comments on individual keywords>

  <section|Inlining>

  <section|Code generation>

  <section|Notes>

  <\itemize>
    <item>Endow the <with|font family|tt|lazy_rep> class with an extra field
    <with|font family|tt|generic expect>, which is the expected type of the
    compiled expression. This can be used both to speed up compilation and to
    have an easy to use type filter at the construction level of <with|font
    family|tt|lazy> (think of <with|font family|tt|while>). It may also be
    used to automatically determine the return types of certain <with|font
    family|tt|lambda> expressions and the like. The <with|font
    family|tt|expect> field may also contain <with|font family|tt|"type!">
    (or maybe <with|font family|tt|()>), in which case we expect a type
    expression.

    <item>Endow the <with|font family|tt|context_rep> class with a field
    <with|font family|tt|int kind> (declare, local, lambda, etc.) and with a
    field <with|font family|tt|generic ret_type>, which contains the desired
    type of a <with|font family|tt|return> for the current block. This type
    may sometimes be determined in advance and sometimes only ad hoc. A flag
    <with|font family|tt|bool <format|no line break>adhoc> may be useful to
    determine in which situation we are.
  </itemize>

  \;
</body>

<\initial>
  <\collection>
    <associate|odd page margin|30mm>
    <associate|paragraph width|150mm>
    <associate|page right margin|30mm>
    <associate|page top margin|30mm>
    <associate|reduction page right margin|25mm>
    <associate|paragraph hyphenation|professional>
    <associate|reduction page bottom margin|15mm>
    <associate|page type|a4>
    <associate|reduction page left margin|25mm>
    <associate|even page margin|30mm>
    <associate|page bottom margin|30mm>
    <associate|reduction page top margin|15mm>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|cond-exports|4.5>
    <associate|rem-spec|6.1>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate>
      toc
    <|associate>
      <vspace*|1fn><with|font series|<quote|bold>|1 Survey of the
      compiler><htab|5mm>?<vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|2 The
      parser><htab|5mm>?<vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|3 Semantic
      analysis><htab|5mm>?<vspace|0.5fn>

      3.1 The current environment<htab|5mm>?

      3.2 Definition of a set of mutually dependent variables and
      constants<htab|5mm>?

      3.3 Importation<htab|5mm>?

      3.4 Type satisfaction<htab|5mm>?

      3.5 Implicit type conversion<htab|5mm>?

      <vspace*|1fn><with|font series|<quote|bold>|4
      Inlining><htab|5mm>?<vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|5 Code
      generation><htab|5mm>?<vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|6
      Notes><htab|5mm>?<vspace|0.5fn>
    </associate>
  </collection>
</auxiliary>
