
/******************************************************************************
* MODULE     : lazy_compound.cc
* DESCRIPTION: Lazy compilation of compounds
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <lazy.hh>

class lazy_compound_rep: public lazy_rep {
public:
  lazy_compound_rep (context con, generic src, list subs);
  list compute_match (list expected_types, list args);
  list compute_matches (list func, list args);
  list compute_matches ();
  bool compile_this (bool force);
};

lazy_compound_rep::lazy_compound_rep (context con, generic g, list subs):
  lazy_rep (con, g, subs) {}

list
lazy_compound_rep::compute_match (list expected, list args) {
  // cout << "Match " << expected << ", " << args << lf;

  if (nil (expected)) return list ();
  list next= compute_match (cdr (expected), cdr (args));
  if ((!atom (expected)) && nil (next)) return next;

  generic type  = car (expected);
  if (is_func (type, DECLARATION, 3)) type= type[2];
  list arg_ms= as_lazy (car (args))->meanings;
  while (!nil (arg_ms)) {
    if (car (arg_ms) [1] == type)
      return cons (car (arg_ms) [0], next);
    arg_ms= cdr (arg_ms);
  }
  return list ();
}

static list
get_tuple (generic g) {
  if (nil (g) || (g == TUPLE)) return list ();
  if ((N(g) > 0) && (g[0] == TUPLE)) return cdr (as_list (g));
  return list (g);
}

list
lazy_compound_rep::compute_matches (list f_ms, list args) {
  // f_ms: possible meanings of the function
  // args: list of compiled arguments
  // computes possible meanings of f (args)

  // cout << "Matches " << f_ms << ", " << args << lf;  
  list r;
  while (!nil (f_ms)) {
    generic f   = car (f_ms) [0];
    generic type= car (f_ms) [1];
    if ((N(type) == 3) && (type[0] == INTO)) {
      list expected= get_tuple (type[1]);
      if (N(expected) == N(args)) {
	list g= compute_match (expected, args);
	if ((nil (args)) || (!nil (g)))
	  r= cons (list (cons (f, g), type[2]), r);
      }
    }
    f_ms= cdr (f_ms);
  }
  return r;
}

list
lazy_compound_rep::compute_matches () {
  list l= compute_matches (as_lazy (car (subs))->meanings, cdr (subs));
  if (nil (l)) {
    ::dereference (subs);
    l= compute_matches (as_lazy (car (subs))->meanings, cdr (subs));
  }
  return l;
}

bool
lazy_compound_rep::compile_this (bool force) { (void) force;
  list l= compute_matches ();
  if (nil (l)) error ("type mismatch in function application", src);
  else if (!atom (l)) error ("ambiguous function application", src);
  else meaning (car (l) [0], car (l) [1]);
  return TRUE;
}

generic
make_lazy_compound (generic g) {
  context con= as_context (g[0]);
  return lazy (new lazy_compound_rep (con, g[1], as_list (g[1])));
}
