
/******************************************************************************
* MODULE     : lazy_declaration.cc
* DESCRIPTION: Lazy compilation of declarations
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <lazy.hh>

class lazy_declaration_rep: public lazy_rep {
  generic var;
public:
  lazy_declaration_rep (context con, generic src,
			generic var, generic type, generic init);
  void notify_compiled (int arg);
  bool compile_this (bool force);
};

lazy_declaration_rep::lazy_declaration_rep (
  context con, generic g, generic var2, generic type, generic init):
    lazy_rep (con, g, nil (init)? list (type): list (type, init)),
    var (var2) {}

void
lazy_declaration_rep::notify_compiled (int arg) {
  if (arg == 0) S(var)[1]= get_value (S(var)[1]);
  if (arg == 1) S(var)[2]= get_value (S(var)[2]);
}

bool
lazy_declaration_rep::compile_this (bool force) { (void) force; 
  meaning (list (DECLARATION, var, S(var)[1], S(var)[2]), con->get_void ());
  return TRUE;
}

generic
retrieve_var (generic g) {
  if (is_func (g, ASSIGN, 2)) return retrieve_var (g[1]);
  if (is_func (g, DEFINE, 2)) return retrieve_var (g[1]);
  if (is_func (g, OFTYPE, 2)) return retrieve_var (g[1]);
  if ((N(g)>0) && (g[0] != TUPLE)) return retrieve_var (g[0]);
  return g;
}

generic
retrieve_type (generic g) {
  if (is_func (g, ASSIGN, 2)) return list (ALIAS_T, retrieve_type (g[1]));
  if (is_func (g, DEFINE, 2)) return retrieve_type (g[1]);
  if (is_func (g, OFTYPE, 2)) {
    if ((N(g[1])>0) && (g[1][0] != TUPLE)) {
      generic lhs= car (g[1]);
      generic ths= cons (TUPLE, cdr (as_list (g[1])));
      generic rhs= list (INTO, ths, g[2]);
      return retrieve_type (list (OFTYPE, lhs, rhs));
    }
    else return g[2];
  }
  return generic ();
}

generic
retrieve_init (generic lhs, generic rhs) {
  if ((N(lhs) == 0) || (lhs[0] == TUPLE)) return rhs;
  generic args= cons (TUPLE, cdr (lhs));
  generic lambda= list (LAMBDA, args, rhs);
  return retrieve_init (car (lhs), lambda);
}

generic
retrieve_init (generic g) {
  if (is_func (g, ASSIGN, 2) || is_func (g, DEFINE, 2)) {
    if (!is_func (g[1], OFTYPE, 2)) return generic ();
    generic r= retrieve_init (g[1][1], g[2]);
    if (is_func (g, ASSIGN, 2)) r= list (REF, r);
    return r;
  }
  return generic ();
}

generic
make_lazy_declaration (context con, generic g) {
  generic var = retrieve_var (g);
  generic type= retrieve_type (g);
  generic init= retrieve_init (g);
  if (nil (var) || nil (type))
    return make_lazy_error (con, "bad declaration", g);

  type= make_lazy (con, type);
  if (!nil (init)) init= make_lazy (con, init);
  var= con->declare (var, type, init);
  return lazy (new lazy_declaration_rep (con, g, var, type, init));
}

/******************************************************************************
* used by lazy_declare, lazy_local and lazy_cross
******************************************************************************/

list
make_lazy_declarations (context con, list l) {
  if (nil (l)) return l;
  return cons (make_lazy_declaration (con, car (l)),
	       make_lazy_declarations (con, cdr (l)));
}

list
make_lazy_locals (context con, list l) {
  if (atom (l)) return list (make_lazy (con, car (l)));
  return cons (make_lazy_declaration (con, car (l)),
	       make_lazy_locals (con, cdr (l)));
}

bool
is_declaration (generic g) {
  if (is_func (g, ASSIGN, 2)) return is_declaration (g[1]);
  if (is_func (g, DEFINE, 2)) return is_declaration (g[1]);
  return is_func (g, OFTYPE, 2);
}

generic
make_lazy_cross_item (context con, generic g) {
  if (is_declaration (g)) return make_lazy_declaration (con, g);
  else return make_lazy (con, g);
}

list
make_lazy_cross_items (context con, list l) {
  if (nil (l)) return l;
  return cons (make_lazy_cross_item (con, car (l)),
	       make_lazy_cross_items (con, cdr (l)));
}
