
/******************************************************************************
* MODULE     : context.cc
* DESCRIPTION: The compiler context
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <context.hh>
#include <lazy.hh>
#include <mirror.hh>

ostream&
operator << (ostream& out, context con) {
  out << "context<" << con->vars << ">";
  return out;
}

int
context_rep::Type () {
  return CONTEXT_TYPE;
}

void
context_rep::Print (ostream& out) {
  out << context (this);
}

generic
context_rep::declare (generic var, generic type, generic init) {
  if (DEBUG_COMPILATION & DEBUG_DECLARATIONS)
    cout << "Declaring " << var << " : " << type << " := " << init << lf;
  mirror m (list (var, type, init));
  if (!vars->contains (var)) vars (var)= list ();
  vars (var)= cons (m, as_list (vars [var]));
  if (DEBUG_COMPILATION & DEBUG_DECLARATIONS)
    cout << "Vars= " << vars << "\n";
  return m;
}

list
context_rep::get_meanings (generic var) {
  list l= as_list (vars [var]);
  if ((!nil (l)) || nil (master)) return l;
  return master->get_meanings (var);
}

generic
context_rep::get (generic var) {
  list l= get_meanings (var);
  if (nil (l)) return var;
  return car (l);
}

generic
context_rep::get_void () {
  return list (TUPLE);
}

generic
context_rep::get_class () {
  return CLASS_T;
}

generic
context_rep::get_foreign () {
  return FOREIGN_T;
}

generic
context_rep::get_signature () {
  return SIGNATURE_T;
}
