
/******************************************************************************
* MODULE     : context.hh
* DESCRIPTION: The compiler context
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __CONTEXT_HH
#define __CONTEXT_HH
#include <table.hh>

class context_rep;
class context {
protected:
  context_rep* rep;

public:
  inline context (): rep (NULL) {}
  inline context (context_rep* rep2): rep (rep2) {}
  inline context (context master, generic scope);
  inline friend context as_context (generic g) {
    if (nil (g)) return context ();
    if (gentype (g) != CONTEXT_TYPE)
      fatal_error ("generic is not a context", "as_context", "context.hh");
    return context ((context_rep*) g.rep); }
  inline operator generic ();
  inline context_rep* operator -> () { return rep; }

  friend class context_rep;
};

class context_rep: generic_rep {
protected:
  bool    top_level;
  context master;
  generic scope;
  table   vars;

public:
  inline context_rep (context master2, generic scope2):
    top_level (FALSE), master (master2), scope (scope2), vars () {}

  int     Type ();
  void    Print (ostream& out);
  generic declare (generic var, generic type, generic init);
  list    get_meanings (generic var);
  generic get (generic var);

  generic get_void ();
  generic get_class ();
  generic get_foreign ();
  generic get_signature ();

  friend class context;
  friend generic compile (context& con, generic g, list& errors);
  friend generic make_lazy_declare (generic g);
  friend ostream& operator << (ostream& out, context con);
};

inline context::context (context master, generic scope):
  rep (new context_rep (master, scope)) {}
inline context::operator generic () { return generic (rep); }

#endif // __CONTEXT_HH
