
/******************************************************************************
* MODULE     : lazy.hh
* DESCRIPTION: Generic lazy compilation instances
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __LAZY_HH
#define __LAZY_HH
#include <symbol.hh>
#include <context.hh>
#include <compiler.hh>

extern symbol TUPLE, INTO, LAMBDA, REF, DEREF;
extern symbol DECLARATION, ASSIGN, DEFINE, OFTYPE; //, INTEGER, NUMBER, STRING;
extern symbol FOREIGN, IDENTIFIER, COMPOUND; //, CLASS;

extern symbol SIGNATURE_T, CLASS_T, FOREIGN_T;
extern symbol BOOL_T, INT_T, INTEGER_T, STRING_T, NUMBER_T, ALIAS_T;

struct lazy;
struct lazy_rep: generic_rep {
  context con;       // programming context
  generic src;       // source program
  generic req_type;  // required type
  list    subs;      // list of all subprograms to be compiled
  int     status;    // 0: uncompiled, 1: subexprs done, 2: done 3: type OK
  list    meanings;  // possible meanings of src after compilation
  list    errors;    // list of compilation errors

  inline lazy_rep (context con2, generic src2);
  inline lazy_rep (context con2, generic src2, list subs2);

  int Type ();
  void Print (ostream& out);

  void    meaning (generic dest, generic type);
  void    warning (string message, generic where);
  void    error   (string message, generic where);
  generic get_value (generic prg);
  generic get_type (generic prg);
  list    lhs (list l);
  generic rhs (list l);
  list    rht (list l);
  inline  void set_type (generic g);
  virtual void set_types_children ();
  bool    compile (bool force);
  virtual void notify_compiled (int arg);
  virtual bool compile_this (bool force);
  void    dereference ();
  generic filter (generic type);

  friend class lazy;
  friend ostream& operator << (ostream& out, lazy prg);
};

struct lazy {
  lazy_rep* rep;

  inline lazy (lazy_rep* rep2): rep (rep2) {}
  inline lazy (context con, generic src): rep (new lazy_rep (con, src)) {}
  inline friend lazy as_lazy (generic g) {
    if (gentype(g) != LAZY_TYPE)
      fatal_error ("generic is not a lazy", "as_lazy", "lazy.hh");
    return lazy ((lazy_rep*) g.rep); }
  inline operator generic () { return generic (rep); }
  inline lazy_rep* operator -> () { return rep; }
  friend ostream& operator << (ostream& out, lazy prg);

  friend class lazy_rep;
};

void    initialize_lazy ();
generic make_lazy (context con, generic g);
list    make_lazies (context con, list l);
generic make_lazy_error (context con, string message, generic where);
void    dereference (list& l);

inline
lazy_rep::lazy_rep (context con2, generic src2):
  con (con2), src (src2), subs (),
  status (0), meanings (), errors () {}

inline
lazy_rep::lazy_rep (context con2, generic src2, list subs2):
  con (con2), src (src2), subs (make_lazies (con, subs2)),
  status (0), meanings (), errors () {}

inline void
lazy_rep::set_type (generic g) {
  req_type= make_lazy (con, g);
  set_types_children ();
}

#endif // __LAZY_HH
