
/******************************************************************************
* MODULE     : foreign.cc
* DESCRIPTION: System routines for the Mathemagix interpreter
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <foreign.hh>
#include <mm_int.hh>
#include <mm_ostream.hh>

/******************************************************************************
* Booleans
******************************************************************************/

generic
bool_not (generic g) {
  return mm_int (!as_mm_int (g[1]));
}

generic
bool_or (generic g) {
  return mm_int (as_mm_int (g[1]) | as_mm_int (g[2]));
}

generic
bool_xor (generic g) {
  return mm_int (as_mm_int (g[1]) ^ as_mm_int (g[2]));
}

generic
bool_and (generic g) {
  return mm_int (as_mm_int (g[1]) & as_mm_int (g[2]));
}

generic
bool_implies (generic g) {
  return mm_int (as_mm_int (g[2]) | (!as_mm_int (g[2])));
}

generic
bool_equivalent (generic g) {
  return mm_int (as_mm_int (g[1]) == as_mm_int (g[2]));
}

generic
bool_equal (generic g) {
  return mm_int (as_mm_int (g[1]) == as_mm_int (g[2]));
}

generic
bool_not_equal (generic g) {
  return mm_int (as_mm_int (g[1]) != as_mm_int (g[2]));
}

generic
bool_output (generic g) {
  mm_ostream mm_out= as_mm_ostream (g[1]);
  ostream& out (mm_out->out);
  if (as_mm_int (g[2])) out << "true"; else out << "false";
  return mm_out;
}

/******************************************************************************
* Integers
******************************************************************************/

generic
int_plus (generic g) {
  return mm_int (as_mm_int (g[1]) + as_mm_int (g[2]));
}

generic
int_minus (generic g) {
  return mm_int (as_mm_int (g[1]) - as_mm_int (g[2]));
}

generic
int_times (generic g) {
  return mm_int (as_mm_int (g[1]) * as_mm_int (g[2]));
}

generic
int_equal (generic g) {
  return mm_int (as_mm_int (g[1]) == as_mm_int (g[2]));
}

generic
int_not_equal (generic g) {
  return mm_int (as_mm_int (g[1]) != as_mm_int (g[2]));
}

generic
int_less (generic g) {
  return mm_int (as_mm_int (g[1]) < as_mm_int (g[2]));
}

generic
int_less_equal (generic g) {
  return mm_int (as_mm_int (g[1]) <= as_mm_int (g[2]));
}

generic
int_greater (generic g) {
  return mm_int (as_mm_int (g[1]) > as_mm_int (g[2]));
}

generic
int_greater_equal (generic g) {
  return mm_int (as_mm_int (g[1]) >= as_mm_int (g[2]));
}

generic
int_output (generic g) {
  mm_ostream mm_out= as_mm_ostream (g[1]);
  ostream& out (mm_out->out);
  out << as_mm_int (g[2]);
  return mm_out;
}

/******************************************************************************
* Strings
******************************************************************************/

generic
string_concat (generic g) {
  return string (as_string (g[1]) * as_string (g[2]));
}

generic
string_equal (generic g) {
  return mm_int (as_string (g[1]) == as_string (g[2]));
}

generic
string_not_equal (generic g) {
  return mm_int (as_string (g[1]) != as_string (g[2]));
}

generic
string_less (generic g) {
  return mm_int (as_string (g[1]) < as_string (g[2]));
}

generic
string_less_equal (generic g) {
  return mm_int (as_string (g[1]) <= as_string (g[2]));
}

generic
string_greater (generic g) {
  return mm_int (as_string (g[2]) < as_string (g[1]));
}

generic
string_greater_equal (generic g) {
  return mm_int (as_string (g[2]) <= as_string (g[1]));
}

generic
string_output (generic g) {
  mm_ostream mm_out= as_mm_ostream (g[1]);
  ostream& out (mm_out->out);
  string s= as_string (g[2]);
  int i, n= N(s);
  for (i=0; i<n; i++) out << s[i];
  return mm_out;
}
