
/******************************************************************************
* MODULE     : interpreter.hh
* DESCRIPTION: the mathemagix interpreter
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __INTERPRETER_HH
#define __INTERPRETER_HH
#include <string.hh>
#include <compiler.hh>

class interpreter_rep;
class interpreter {
protected:
  interpreter_rep* rep;

public:
  inline interpreter ();
  inline interpreter (interpreter_rep* rep2): rep (rep2) {}
  inline friend interpreter as_interpreter (generic g) {
    if (nil (g)) return interpreter ();
    if (gentype (g) != INTERPRETER_TYPE)
      fatal_error ("generic is not an interpreter",
		   "as_interpreter", "interpreter.hh");
    return interpreter ((interpreter_rep*) g.rep); }
  inline operator generic ();
  inline interpreter_rep* operator -> () { return rep; }

  friend class interpreter_rep;
};

class interpreter_rep: generic_rep {
public:
  context con;
  table   vars;
  table   keyword;
  table   foreign;
  list    local;

public:
  interpreter_rep ();
  void make_keyword (string s, generic (*fn) (generic));
  void make_foreign (string s);
  void make_foreign (string s, ostream& out);
  void make_foreign (string s, generic (*fn) (generic));

  int  Type ();
  void Print (ostream& out);

  void local_start ();
  void local_set (generic var, generic val);
  generic local_end (generic result);

  friend class interpreter;
};

inline interpreter::interpreter (): rep (new interpreter_rep ()) {}
inline interpreter::operator generic () { return generic (rep); }

generic interpret (generic g);
generic compile (string s);
generic compile (generic g);
bool    run (string s);
bool    run (generic g);

#endif // __INTERPRETER_HH
