
/******************************************************************************
* MODULE     : file.cc
* DESCRIPTION: Loading and saving files
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <file.hh>
#include <fstream.h>
#include <list.hh>
#include <stdio.h>
#include <stdlib.h>

/******************************************************************************
* Location handling
******************************************************************************/

static int  cur_location = 0;
static list sources;

void
insert_source (string& src, string name) {
  src->begin= cur_location;
  cur_location += N(src);
  src->end= cur_location;
  cur_location ++;
  sources= cons (list (string (name), src), sources);
}

void
find_location (int location, string& src, string& name, int& row, int& col) {
  list l= sources;

  while (!nil (l)) {
    src= as_string (car (l) [1]);
    if ((location >= src->begin) && (location <= src->end)) {
      int i, pos= location - src->begin;
      name= as_string (car (l) [0]);
      for (i=0, row=1, col=1; i<pos; i++)
	if (src[i]=='\n') { row++; col=1; }
	else col++;
      return;
    }
    l= cdr (l);
  }
 
  src = "";
  name= string ("unknown");
  row = 0;
  col = 0;
}

/******************************************************************************
* Loading and saving
******************************************************************************/

string eval_system (string s);

bool
load (string file_name, string& s, bool decode) {
  if (decode) file_name= eval_system ("echo " * file_name);
  ifstream f1 (as_charp (file_name));
  if (!f1) return TRUE;
  else {
    char c;
    s= string ("");
    while (f1.get (c)) s << c;
    insert_source (s, file_name);
    return FALSE;
  }
}

bool
save (string file_name, string s, bool decode) {
  if (decode) file_name= eval_system ("echo " * file_name);
  ofstream f1 (as_charp (file_name));
  if (!f1) return TRUE;
  else {
    int i, n=N(s);
    for (i=0; f1 && (i<n); i++) f1.put (s[i]);
    return FALSE;
  }
}

/******************************************************************************
* System commands
******************************************************************************/

string
get_temp_file_name () {
  char buffer [L_tmpnam+1];
  if (tmpnam (buffer) == NULL)
    fatal_error ("Couldn't create a temporary file name",
		 "get_temp_file_name", "file.gen.cc");
  return string (buffer);
}

int
system (string s) {
  char* _s= as_charp (s);
  int r= system (_s);
  delete[] _s;
  return r;
}

string
eval_system (string s) {
  string temp= get_temp_file_name ();
  system (s * " > " * temp);
  string r;
  bool flag= load (temp, r, FALSE);
  system ("rm " * temp);
  if (flag) return "";
  while ((N(r)>0) && (r[N(r)-1]=='\n')) r= r (0, N(r)-1);
  return r;
}
