
/******************************************************************************
* MODULE     : array.hh
* DESCRIPTION: arrays
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __ARRAY_HH
#define __ARRAY_HH
#include <generic.hh>

class array_rep: generic_rep {
protected:
  int n;
  generic* a;

  int Type ();
  int Hash ();
  bool Equal (generic x);
  generic Copy ();
  int Arity ();
  generic& Access (int i);
  generic Range (int begin, int end);
  generic& Car ();
  generic Cdr ();
  void Print (ostream& out);

public:
  inline array_rep (): n(0), a(NULL) {}
         array_rep(int n);
  inline ~array_rep () { if (n!=0) delete[] a; }
  void resize (int n2);

  friend class array;
  friend int N (array a);
  friend array copy (array a);
};

class array {
protected:
  array_rep* rep;
  inline array (array_rep* rep2): rep (rep2) {}

public:
  inline array (int n=0): rep (new array_rep (n)) {}
  array (generic *a, int n);
  array (generic g1);
  array (generic g1, generic g2);
  array (generic g1, generic g2, generic g3);
  array (generic g1, generic g2, generic g3, generic g4);
  inline friend array as_array (generic g) {
    if (gentype(g) != ARRAY_TYPE)
      fatal_error ("generic is not an array", "as_array", "array.hh");
    return array ((array_rep*) g.rep); }
  inline operator generic () { return generic (rep); }
  inline array_rep* operator -> () { return rep; }

  inline friend int N (array a) { return a.rep->n; }
  inline generic& operator [] (int i) { return rep->a[i]; }
  inline array operator () (int start, int end) {
    return array (rep->a + start, end-start); }
  friend bool operator == (array a, array b);
  friend bool operator != (array a, array b);
  inline friend array copy (array a) { return array (a->a, a->n); }
  friend ostream& operator << (ostream& out, array a);

  friend class array_rep;
};

array& operator << (array& a, generic x);
array& operator << (array& a, array b);

#endif // __ARRAY_HH
