
/******************************************************************************
* MODULE     : generic.hh
* DESCRIPTION: Abstract types
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __GENERIC_HH
#define __GENERIC_HH
#include <gc.h>
#include <iostream.h>

#ifndef TRUE
#define TRUE  1
#define FALSE 0
#endif

enum indent_t { indent, unindent, lf, pritty, normal };
ostream& operator << (ostream& out, indent_t x);

class generic;

#define UNKNOWN_TYPE       0
#define NIL_TYPE           1
#define STRING_TYPE        2
#define SYMBOL_TYPE        3
#define ARRAY_TYPE         4
#define LIST_TYPE          5
#define TABLE_TYPE         6
#define FUNCTION_TYPE      7
#define MIRROR_TYPE        8
#define CONTEXT_TYPE       9
#define LAZY_TYPE         10
#define ERROR_TYPE        11

#define INTERPRETER_TYPE  12
#define MM_ISTREAM_TYPE   13
#define MM_OSTREAM_TYPE   14
#define MM_INT_TYPE       15
#define MM_DOUBLE_TYPE    16
#define MM_INTEGER_TYPE   17

struct generic_rep {
  int begin, end;
  inline generic_rep (int b=-1, int e=-1): begin (b), end (e) {}
  inline generic_rep (generic g);
  inline virtual ~generic_rep () {}

  virtual int Type ();
  virtual int Hash ();
  virtual bool Equal (generic g);
  virtual generic Copy ();
  virtual int Arity ();
  virtual generic& Access (int i);
  virtual generic Range (int begin, int end);
  virtual generic& Car ();
  virtual generic Cdr ();
  virtual void Print (ostream& out);
};

struct generic {
  generic_rep* rep;

  inline generic (): rep (NULL) {}
  inline generic (generic_rep* rep2): rep (rep2) {}
  inline generic_rep* operator -> () { return rep; }
  inline bool operator == (generic g) { return rep->Equal (g); }
  inline bool operator != (generic g) { return !rep->Equal (g); }
  inline generic& operator [] (int i) { return rep->Access (i); }
  inline generic operator () (int beg, int end) {
    return rep->Range (beg, end); }
};

inline generic_rep::generic_rep (generic g) {
  if (g.rep != NULL) { begin= g->begin; end= g->end; } }
inline bool nil (generic g) { return g.rep==NULL; }
inline int gentype (generic g) {
  if (nil (g.rep)) return NIL_TYPE;
  return g.rep->Type (); }
inline int hash (generic g) {
  if (nil (g.rep)) return 0;
  return g.rep->Hash (); }
inline int N (generic g) {
  if (nil (g.rep)) return 0;
  return g.rep->Arity (); }
inline generic& car (generic g) { return g.rep->Car (); }
inline generic cdr (generic g) { return g.rep->Cdr (); }
inline ostream& operator << (ostream& out, generic g) {
  if (nil (g)) return out << "()";
  g.rep->Print (out);
  return out; }
void print_entries (ostream& out, generic g, bool flag= FALSE);
generic& S (generic g);
generic extend (generic g, generic with); // really generic& g

volatile inline void fatal_error (char* message, char* routine, char* file) {
  cout << "\nFatal error: " << message
       << "\n             in '" << routine
       << "' in file '" << file << "'\n";
  exit (1);
}

inline void* operator new   (register size_t sz) { return GC_MALLOC (sz); }
inline void* operator new[] (register size_t sz) { return GC_MALLOC (sz); }
inline void operator delete (register void* ptr)   { (void) ptr; }
inline void operator delete[] (register void* ptr) { (void) ptr; }

#endif // __GENERIC_HH
