
/******************************************************************************
* MODULE     : symbol.cc
* DESCRIPTION: Fixed size symbols. Zero characters can be part of symbol.
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <symbol.hh>

/******************************************************************************
* Required functions for generic structures
******************************************************************************/

int
symbol_rep::Type () {
  return SYMBOL_TYPE;
}

int
symbol_rep::Hash () {
  return hash_value;
}

bool
symbol_rep::Equal (generic g) {
  if (gentype (g) != SYMBOL_TYPE) return FALSE;
  return symbol (this) == as_symbol (g);
}

generic
symbol_rep::Copy () {
  return copy (symbol (this));
}

void
symbol_rep::Print (ostream& out) {
  out << symbol (this);
}

/******************************************************************************
* Constructors and convertors for symbols
******************************************************************************/

static table symbol_table;

inline
symbol_rep::symbol_rep (generic g, int begin, int end):
  generic_rep (begin, end),
  hash_value (hash(g)^0x55555555)
{
  generic a= symbol_table [g];
  ref= &a[0];
}

symbol::symbol (char* s, int begin, int end) {
  generic g= string (s);
  if (!symbol_table->contains (g))
    symbol_table (g)= list (g);
  rep= new symbol_rep (g, begin, end);
}

symbol::symbol (string s, int begin, int end) {
  generic g= s;
  if (!symbol_table->contains (g))
    symbol_table (g)= list (g);
  rep= new symbol_rep (g, begin, end);
}

symbol::symbol (generic g, int begin, int end) {
  if (!symbol_table->contains (g))
    symbol_table (g)= list (g);
  rep= new symbol_rep (g, begin, end);
}

symbol::symbol (char* s, generic where) {
  generic g= string (s);
  if (!symbol_table->contains (g))
    symbol_table (g)= list (g);
  if (nil (where)) rep= new symbol_rep (g);
  else rep= new symbol_rep (g, where->begin, where->end);
}

symbol::symbol (string s, generic where) {
  generic g= s;
  if (!symbol_table->contains (g))
    symbol_table (g)= list (g);
  if (nil (where)) rep= new symbol_rep (g);
  else rep= new symbol_rep (g, where->begin, where->end);
}

symbol::symbol (generic g, generic where) {
  if (!symbol_table->contains (g))
    symbol_table (g)= list (g);
  if (nil (where)) rep= new symbol_rep (g);
  else rep= new symbol_rep (g, where->begin, where->end);
}

symbol::symbol (string s) {
  generic g= string (s);
  if (!symbol_table->contains (g))
    symbol_table (g)= list (g);
  rep= new symbol_rep (g, g->begin, g->end);
}

symbol::symbol (generic g) {
  if (!symbol_table->contains (g))
    symbol_table (g)= list (g);
  if (nil (g)) rep= new symbol_rep (g, -1, -1);
  else rep= new symbol_rep (g, g->begin, g->end);
}

ostream&
operator << (ostream& out, symbol s) {
  generic g= (*s.rep->ref);
  if (gentype (g) == STRING_TYPE) {
    string s= as_string (g);
    int i, n=N(s);
    for (i=0; i<n; i++) out << s[i];
  }
  else out << "#symbol<" << g << ">";
  return out;
}
