
/******************************************************************************
* MODULE     : table.cc
* DESCRIPTION: tables
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <table.hh>

/******************************************************************************
* Required functions for generic structures
******************************************************************************/

int
table_rep::Type () {
  return TABLE_TYPE;
}

int
table_rep::Hash () {
  register int i, h=0;
  for (i=0; i<n; i++) {
    h = (h<<11) + (h>>21);
    h = h + hash (a[i]);
  }
  return h;
}

bool
table_rep::Equal (generic g) {
  if (gentype (g) != TABLE_TYPE) return FALSE;
  return table (this) == as_table (g);
}

generic
table_rep::Copy () {
  return copy (table (this));
}

int
table_rep::Arity () {
  return size;
}

void
table_rep::Print (ostream& out) {
  out << table (this);
}

/******************************************************************************
* Routines for table representations
******************************************************************************/

void
table_rep::resize (int n2) {
  int i;
  int oldn= n;
  list* olda= a;
  n= n2;
  a= new list[n];
  for (i=0; i<oldn; i++) {
    list l (olda[i]);
    while (!nil (l)) {
      list& newl (a [hash(car(l)[0]) & (n-1)]);
      newl= cons (car(l), newl);
      l= cdr(l);
    }
  }
  delete[] olda;
}

bool
table_rep::contains (generic g) {
  list l (a [hash(g) & (n-1)]);
  while (!nil (l)) {
    if (car(l)[0] == g) return TRUE;
    l= cdr(l);
  }
  return FALSE;
}

void
table_rep::reset (generic g) {
  list* l= &(a[hash(g) & (n-1)]);
  while (!nil (*l)) {
    if (car(*l)[0] == g) {
      *l= cdr(*l);
      size --;
      if (size < (n>>2)) resize (n>>1);
      return;
    }
    l= &cdr(*l);
  }
}

list
table_rep::entries () {
  list r;
  int i;
  for (i=0; i<n; i++) {
    list l (a[i]);
    while (!nil (l)) {
      r= cons (car (l), r);
      l= cdr (l);
    }
  }
  return reverse (r);
}

generic&
table_rep::bracket_rw (generic g) {
  list l (a [hash(g) & (n-1)]);
  while (!nil (l)) {
    if (car(l)[0] == g) return car(l)[1];
    l= cdr(l);
  }
  if (size >= (n>>1)) resize (n<<1);
  list& rl (a [hash(g) & (n-1)]);
  rl= cons (list (g, generic()), rl);
  size ++;
  return car(rl)[1];
}

generic
table_rep::bracket_ro (generic g) {
  list  l (a [hash(g) & (n-1)]);
  while (!nil (l)) {
    if (car(l)[0] == g) return car(l)[1];
    l= cdr(l);
  }
  return generic ();
}

/******************************************************************************
* Routines for tables
******************************************************************************/

ostream&
operator << (ostream& out, table t) {
  out << "#table";
  print_entries (out, t.rep->entries ());
  return out;
}

table
copy (table t1) {
  int i, n= t1.rep->size;
  table t2 (n);
  for (i=0; i<n; i++)
    t2.rep->a[i]= copy (t1.rep->a[i]);
  return t2;
}

bool
operator == (table t1, table t2) {
  int i, n= t1.rep->size;
  if (t2.rep->size != n) return FALSE;
  for (i=0; i<n; i++) {
    list l (t1.rep->a[i]);
    while (!nil (l)) {
      if (!t2->contains (car(l)[0])) return FALSE;
      if (t2[car(l)[0]] != car(l)[1]) return FALSE;
      l= cdr (l);
    }
  }
  return TRUE;
}

bool
operator != (table t1, table t2) {
  return !(t1==t2);
}
