
/******************************************************************************
* MODULE     : table.hh
* DESCRIPTION: tables
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __TABLE_HH
#define __TABLE_HH
#include <list.hh>

class table;
class table_rep: generic_rep {
protected:
  int size;   // size of table (nr of entries)
  int n;      // nr of keys (a power of two)
  list* a;    // the array of entries

  int Type ();
  int Hash ();
  bool Equal (generic g);
  generic Copy ();
  int Arity ();
  void Print (ostream& out);

public:
  inline table_rep (int n2): size (0), n (n2), a (new list [n]) {}
  inline ~table_rep () { delete[] a; }
  void resize (int n);
  bool contains (generic g);
  void reset (generic g);
  list entries ();
  generic  bracket_ro (generic g);
  generic& bracket_rw (generic g);

  friend class table;
  friend int N (table a);
  friend table copy (table t);
  friend bool operator == (table t1, table t2);
};

class table {
protected:
  table_rep* rep;
  inline table (table_rep* rep2): rep (rep2) {}

public:
  inline table (int n=2): rep (new table_rep (n)) {}
  inline friend table as_table (generic g) {
    if (gentype(g) != TABLE_TYPE)
      fatal_error ("generic is not a table", "as_table", "table.hh");
    return table ((table_rep*) g.rep); }
  inline operator generic () { return generic (rep); }
  inline table_rep* operator -> () { return rep; }

  inline friend int N (table t) { return t.rep->size; }
  inline generic  operator [] (generic g) { return rep->bracket_ro (g); }
  inline generic& operator () (generic g) { return rep->bracket_rw (g); }
  friend bool operator == (table a, table b);
  friend bool operator != (table a, table b);
  friend table copy (table a);
  friend ostream& operator << (ostream& out, table a);

  friend class table_rep;
};

#endif // __TABLE_HH
